// Firmware: "Der heie Draht"
// Kurzbezeichnung: DHD 1
// Compiler: IAR Embended Workbench 4.3
// kostenlose 4K-Version erhhltlich unter
// http://supp.iar.com/Download/SW/?item=EWAVR-KS4
// (Anmeldung erforderlich)
//
// letzte nderung:  2.3.2010
// Controller: ATmega8 (Dip-Gehuse)
// Clockfrequnez = 4 MHz extern
// Author: ELV
// Version: 1.0
// Fuse-Bits:
// CKSEL=1100 SUT=00   (8MHz)


#include "iom8.h"
#include "stdio.h"
#include "ioavr.h"
#include "inavr.h"
#include "ina90.h"


#define SPEED           120         // Wartezeit fr den Zhler(uiCounter)
#define Start !(PINB & (1<<PB1))    // Ports definieren
#define Fehler!(PINB & (1<<PB4))
#define Stop  !(PINB & (1<<PB2))
#define Eingang4 !(PINB & (1<<PB0)) // nicht benutzt

// Zuordnung Dezimalzahlen -> 7-Segmentanzeige
//                                   0    1    2    3    4    5    6    7    8    9         -    F    A    U
unsigned char __flash ucCharSet[]   = {0xC0,0xF9,0xA4,0xB0,0x99,0x92,0x82,0xF8,0x80,0x90,0xFF,0xBF,0x8E,0x88,0xC1};

//  hier sind die Tonfolgen (Bytewerte fr die Frequenzen /Tonfolge) gespeichert
unsigned char __flash ucSoundSet1[] = {0xff,0xff,0x00,0xff,0xff,0x00,0xee,0x0,0xee,0x0,0xdd,0x0,0xdd,0xcc,0x0,0xbb};
unsigned char __flash ucSoundSet2[] = {0xff,0xe0,0xd0,0xc0,0xb0,0xa0,0x90,0x00,0x00,0x00};

unsigned char ucDisplayMemory[3];  // Displayspeicher
unsigned int  uiWait=SPEED;
unsigned char ucBlinken =0;        // Flag fr Blinken
unsigned int  uiCounter =0;        // Zhler fr bentigte Zeit
unsigned char ucStart;             // Flag ucStart
unsigned int  uiHighscore=0;       // Speicher fr Highscore

//******************************************************************************
// Initialisierung
//

void init(void)
{
   //Port Installation
   //DDRn:  1 -> Ausgang 0 -> Eingang
   //PORTn: 1 -> Pullup  0 -> Kein Pullup

   // Port B
   PORTB = 0x1F;
   DDRB  = (0<<DDB0) | (0<<DDB1) | (0<<DDB2) | (1<<DDB3) | (0<<DDB4) | (0<<DDB5) | (0<<DDB6) | (0<<DDB7);

   // Port C
   PORTC = 0x00;         // keine Pullups
   DDRC  = 0xff;         // alle Ports als Ausgang

   // Port D
   PORTD = 0x00;         // keine Pullups
   DDRD  = 0xFF;         // alle Ports als Ausgang

   // Timer 0 initialisieren
   TCCR0 = 0x03;        // Prescaler = 64
   TIMSK = (1<<TOIE0);  // Interrupt bei Timerberlauf von Timer0


   // Timer 1 initialisieren
   TCCR1B = (1<<CS12)| (0<<CS11)| (1<<CS10);  // Prescaler = 1024

   // Timer 2 initialisieren
   // als Frequenzgenerator  OCR2 -> Frequenz
   TCCR2= (1<<WGM20)|(0<<COM21) | (1<<COM20)| (1<<WGM21)| (0<<CS22)| (1<<CS21)| (0<<CS20);

   _SEI();             // Globalen Interrupt freigeben
}
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interruptroutine : Timer0 berlauf
// wird bei jedem Timer0 berlauf aufgerufen

#pragma vector=TIMER0_OVF_vect
__interrupt void scheduler(void)
{
   static unsigned char ucMultiplexCnt=0;        // Zhlerfr Displaystelle
   if(uiWait)
      uiWait--;

   if (uiWait==0)                                // Warteschleife
   {
      if ((uiCounter<1000)&( ucStart))uiCounter++;  //Zhler erhhen
      uiWait=SPEED;
   }

//***************************************************************************
// Displaymultiplex
//
   PORTC |= 0x0F;                                // Alle Segmente ausschalten
   if ((TCNT1 & 0x800) && ucBlinken)PORTD=0xFF;  // Blinken aktivieren  falls
   else PORTD = ucDisplayMemory[ucMultiplexCnt]; // gesetzt (Hihscore)

// Zahlenwerte ausgeben bei jedem Interrupt nur eine Stelle!
   if (ucMultiplexCnt ==0)PORTC = 0x0E;          // Digit DI1 aktivieren
   if (ucMultiplexCnt ==1)PORTC = 0x0D;          // Digit DI2 aktivieren
   if (ucMultiplexCnt ==2)PORTC = 0x0B;          // Digit DI3 aktivieren
   if(ucMultiplexCnt < 2)                        // Digit erhhen
      ucMultiplexCnt++;
   else
      ucMultiplexCnt=0;

}


//******************************************************************************
// Zeige Wert zwischen  0 and 999
// uiValue = bergabewert

void show_value(unsigned int uiValue)
{
   unsigned char ucDigit;
   unsigned char ucZero=1;                        // 1 = nullen ausblenden

   if(uiValue < 1000)                             // bergabewert < 1000?
   {                                              // ja, dann hier weiter
      ucDigit = (unsigned char)(uiValue / 100);   // zeige 100er
      if((ucZero == 1) && (ucDigit == 0))         // 0 ausblenden
         ucDigit = 0x0A;                          // Leerzeichen
      else
         ucZero = 0;
      ucDisplayMemory[1] = ucCharSet[ucDigit];
      uiValue %= 100;

      ucDigit = (unsigned char)(uiValue / 10);    // zeige 10er
      if((ucZero == 1) && (ucDigit == 0) )        // 0 ausblenden
         ucDigit = 0x0A;                          // Leerzeichen
      ucDisplayMemory[2] = ucCharSet[ucDigit];

      ucDigit = (unsigned char)(uiValue % 10);    // zeige 1er
      ucDisplayMemory[0] = ucCharSet[ucDigit];
   }
   else                                           // bergabewert<1000
   {                                              // Nein, dann berlauf
      ucDisplayMemory[0] = ucCharSet[0x0B];       //  --- anzeigen
      ucDisplayMemory[1] = ucCharSet[0x0B];       //
      ucDisplayMemory[2] = ucCharSet[0x0B];       //
   }
}
void OCR2_OFF ()  // Aktiviert die Soundausgabe
{
   TCCR2= (1<<WGM20)|(0<<COM21) | (1<<COM20)| (1<<WGM21)| (0<<CS22)| (1<<CS21)| (0<<CS20);
}
void OCR2_ON ()   // Deaktiviert die Soundausgabe
{
   TCCR2= (0<<WGM20)|(0<<COM21) | (1<<COM20)| (1<<WGM21)| (0<<CS22)| (1<<CS21)| (0<<CS20);
}

void Sound_Peep ()
{
   OCR2_ON();                      // Sound aktivieren
   OCR2=100;                       // Frequenz festlegen 100=> 2kHz
   __delay_cycles(300000);         // Tonlnge
   OCR2_OFF();                     // Sound deaktivieren
}
void Sound_Good ()                 // gibt eine Tonfolge aus
{
   unsigned char i;                // Anzahl der Tne

   OCR2_ON();                      // Sound aktivieren
   for (i=0;i<8;i++)               //
   {
      OCR2= ucSoundSet2[i];        // Werte aus Tabelle holen
      __delay_cycles(500000);      // und ausgeben
   }
   OCR2_OFF();                     // Sound deaktivieren
}

void Sound_Highscore ()            // gibt eine Tonfolge bei einem HighScore aus
{
   unsigned char i;

   OCR2_ON();                      // Soundausgabe aktivieren
   for (i=0;i<16;i++)              // Anzahl der Tne
   {
      OCR2= ucSoundSet1[i];        // Frequenzwert aus der Tabelle hlen
                                   // und ausgeben
      __delay_cycles(500000);      // Warten
   }
   OCR2_OFF();                     // Soundausgabe deaktivieren
}


void Sound_Fehler ()
{  unsigned char i;

OCR2_ON();                          // Soundausgabe aktivieren
for (i=0;i<70;i++)                  // Schleife installieren
{
   OCR2=0x80+i;                     // Frequenz ausgeben und erniedrigen
   __delay_cycles(100000);          // Warten
}
OCR2_OFF();                         // Soundausgabe deaktivieren
}


// ********* Hauptschleife *********
void main()
{
   unsigned int uiTemp;
   init();                          // Ports und Timer aktivieren
   Sound_Peep();                    // erzeuge kurzen Signalton
   show_value(1999);                // Anzeige "---"
   while(1)                         // Endlosschleife
   {
      __delay_cycles(10);           // kurze Wartezeit
      if (Start)                    // Startkontakt aktiviert?
      {                             // ja, dann hier weiter
         ucStart=1;                 // Flag "ucStart"setzen
         Sound_Peep();              // Ton ausgeben
         ucBlinken=0;               // Blinken deaktivieren
         uiCounter=0;               // Zhler "uiCounter" auf null
      }

      if (Fehler)                   // Kontakt geschlossen? (Fehler)
      {                             // ja, dann hier weiter
         if (ucStart)
         {
            Sound_Fehler();         // Fehlerton ausgeben
            show_value(1999);       // Anzeige "---"
            __delay_cycles(1000000);// Warten
         }
         ucStart=0;                 // Flag "ucStart" zurcksetzen
      }

      if (Stop)                     // Kontakt geschlossen? (Ziel erreicht?)
      {
         uiTemp= uiCounter;         // Aktullen Zhlerstand sichern
         if (ucStart && (uiTemp < uiHighscore)) // Vergleichen auf Highscore
         {
            uiHighscore = uiTemp;   // Higscore sichern
            TCNT1=0;                // Timer 1 auf 0 (Blinken)
            ucBlinken =1;           // Blinken aktivieren
            Sound_Highscore();      // Tonfolge fr Highscore ausgeben
         }
         else  Sound_Good();        // kein Highscore, dann normale Tonfolge ausgeben
         __delay_cycles(5000000);   // noch etwas warten
         ucStart=0;                 // Flag zurcksetzen
      }
      if (ucStart) show_value(uiCounter); // Zhlerstand auf Display ausgeben
      if (TCNT1 > 0xFFF0)ucBlinken=0 ;    // Blinken ausschalten
   }
}

